@@ -42,6 +42,10 @@ class MPlotQueriesTool(LogFileTool):
         self.argparser.description='A script to plot various information from logfiles. ' \
             'Clicking on any of the plot points will print the corresponding log line to stdout.'
 
+        # disable some default shortcuts
+        plt.rcParams['keymap.xscale'] = ''
+        plt.rcParams['keymap.yscale'] = ''
+
         # import all plot type classes in plottypes module
         self.plot_types = [c[1] for c in inspect.getmembers(plottypes, inspect.isclass)]
         self.plot_types = dict((pt.plot_type_str, pt) for pt in self.plot_types)
@@ -53,7 +57,8 @@ class MPlotQueriesTool(LogFileTool):
         self.argparser.add_argument('--type', action='store', default='scatter', choices=self.plot_types.keys(), help='type of plot (default=scatter with --yaxis duration).')        
         self.argparser.add_argument('--title', action='store', default=None, help='change the title of the plot (default=filename(s))')        
         self.argparser.add_argument('--group', help="specify value to group on. Possible values depend on type of plot. All basic plot types can group on 'namespace', 'operation', 'thread', 'pattern', range and histogram plots can additionally group on 'log2code'. The group can also be a regular expression.")
-        self.argparser.add_argument('--group-limit', metavar='N', type=int, default=None, help="specify an upper limit of the number of groups. Groups are sorted by number of data points. If limit is specified, only the top N will be listed separately, the rest are grouped together in an 'other' group")
+        self.argparser.add_argument('--group-limit', metavar='N', type=int, default=None, help="specify an upper limit of the number of groups. Groups are sorted by number of data points. If limit is specified, only the top N will be listed separately, the rest are grouped together in an 'others' group")
+        self.argparser.add_argument('--no-others', action='store_true', default=False, help="if this flag is used, the 'others' group (see --group-limit) will be discarded.")
         self.argparser.add_argument('--optime-start', action='store_true', default=False, help="plot operations with a duration when they started instead (by subtracting the duration). The default is to plot them when they finish (at the time they are logged).")
 
         self.legend = None
@@ -109,15 +114,13 @@ class MPlotQueriesTool(LogFileTool):
             self.logfiles = self.args['logfile']
             
         if len(self.logfiles) > 1:
+            # force "logfile" to be the group key for multiple files
             multiple_files = True
             self.args['group'] = 'filename'
         
         plot_instance = self.plot_types[self.args['type']](args=self.args, unknown_args=self.unknown_args)
 
         for logfile in self.logfiles:
-            start = None
-            end = None         
-            logevent = None
             
             # get log file information
             if self.progress_bar_enabled:
@@ -140,9 +143,6 @@ class MPlotQueriesTool(LogFileTool):
                     logevent._datetime = logevent._datetime - timedelta(milliseconds=logevent.duration)
                     logevent._datetime_calculated = True
 
-                if not start:
-                    start = logevent.datetime
-
                 # update progress bar every 1000 lines
                 if self.progress_bar_enabled and (i % 1000 == 0) and logevent.datetime:
                     progress_curr = self._datetime_to_epoch(logevent.datetime)
@@ -165,15 +165,8 @@ class MPlotQueriesTool(LogFileTool):
                     logevent.filename = logfile.name
 
 
-            # store end of logfile 
-            if logevent and logevent.datetime:
-                if self.args['optime_start'] and hasattr(logevent, 'end_datetime'):
-                    end = logevent.end_datetime
-                else:
-                    end = logevent.datetime
-
-            # store start and end for each logfile
-            plot_instance.date_range = (start, end)
+            # store start and end for each logfile (also works for system.profile and stdin stream)
+            plot_instance.date_range = (logfile.start, logfile.end)
 
         # clear progress bar
         if self.logfiles and self.progress_bar_enabled:
@@ -271,19 +264,22 @@ class MPlotQueriesTool(LogFileTool):
 
 
     def print_shortcuts(self):
-        if self.args['type'] == 'scatter':
-            print "keyboard/mouse shortcuts (focus must be on figure window):"
-            print "  click on legend item to toggle individual plots"
-        else:
-            print "keyboard shortcuts (focus must be on figure window):"
+        print "\nkeyboard shortcuts (focus must be on figure window):\n"
+
+        print "    %8s  %s" % ("p", "switch to pan mode")
+        print "    %8s  %s" % ("o", "switch to zoom mode")
+        print "    %8s  %s" % ("left/right", "back / forward")
+        print "    %8s  %s" % ("h", "home (original view)")
+        print "    %8s  %s" % ("l", "toggle log/linear y-axis")
+        print "    %8s  %s" % ("f", "toggle fullscreen")
+        print "    %8s  %s" % ("1-9", "toggle visibility of top 10 individual plots 1-9")
+        print "    %8s  %s" % ("0", "toggle visibility of all plots")
+        print "    %8s  %s" % ("-", "toggle visibility of legend")
+        print "    %8s  %s" % ("g", "toggle grid")
+        print "    %8s  %s" % ("c", "toggle 'created with' footnote")
+        print "    %8s  %s" % ("s", "save figure")
+        print "    %8s  %s" % ("q", "quit mplotqueries")
 
-        print "%5s  %s" % ("1-9", "toggle visibility of individual plots 1-9")
-        print "%5s  %s" % ("0", "toggle visibility of all plots")
-        print "%5s  %s" % ("-", "toggle visibility of legend")
-        print "%5s  %s" % ("f", "toggle visibility of footnote")
-        print "%5s  %s" % ("g", "toggle grid")
-        print "%5s  %s" % ("l", "toggle log/linear y-axis")
-        print "%5s  %s" % ("q", "quit mplotqueries")
         print
 
 
@@ -306,7 +302,7 @@ class MPlotQueriesTool(LogFileTool):
 
         # get PlotType and let it print that event
         plot_type = event.artist._mt_plot_type
-        plot_type.print_line(event)
+        plot_type.clicked(event)
 
 
     def toggle_artist(self, artist):
@@ -319,6 +315,7 @@ class MPlotQueriesTool(LogFileTool):
 
 
     def onpress(self, event):
+        # number keys
         if event.key in ['1', '2', '3', '4', '5', '6', '7', '8', '9']:
             idx = int(event.key)-1
             try:
@@ -326,6 +323,7 @@ class MPlotQueriesTool(LogFileTool):
             except IndexError:
                 pass
 
+        # 0, toggle all plots
         if event.key == '0':
             try:
                 visible = any([a.get_visible() for a in self.artists])
@@ -336,18 +334,32 @@ class MPlotQueriesTool(LogFileTool):
                 artist.set_visible(not visible)
             plt.gcf().canvas.draw()
 
+        # quit
         if event.key == 'q':
             raise SystemExit('quitting.')
 
+        # toggle legend
         if event.key == '-':
             if self.legend:
                 self.toggle_artist(self.legend)
                 plt.gcf().canvas.draw()
 
-        if event.key == 'f':
+        # toggle created footnote
+        if event.key == 'c':
             self.toggle_artist(self.footnote)
             plt.gcf().canvas.draw()
 
+        # toggle yaxis logscale
+        if event.key == 'l':
+            scale = plt.gca().get_yscale()
+            if scale == 'linear':
+                plt.gca().set_yscale('log')
+            else:
+                plt.gca().set_yscale('linear')
+
+            plt.autoscale(True, axis='y', tight=True)
+            plt.gcf().canvas.draw()
+
 
     def plot(self):
         # check if there is anything to plot
