@@ -46,7 +46,9 @@ class TestMLogFilter(object):
         self.tool.run('%s --from %s'%(self.logfile_path, random_start.strftime("%b %d %H:%M:%S")))
         output = sys.stdout.getvalue()
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
+            if not le.datetime:
+                continue
             assert(le.datetime >= random_start)
 
     def test_from_iso8601_timestamp(self):
@@ -54,7 +56,9 @@ class TestMLogFilter(object):
         self.tool.run('%s --from %s'%(self.logfile_path, random_start.isoformat()))
         output = sys.stdout.getvalue()
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
+            if not le.datetime:
+                continue
             assert(le.datetime >= random_start)
 
     def test_from_to(self):
@@ -64,9 +68,27 @@ class TestMLogFilter(object):
         self.tool.run('%s --from %s --to %s'%(self.logfile_path, random_start.strftime("%b %d %H:%M:%S"), random_end.strftime("%b %d %H:%M:%S")))
         output = sys.stdout.getvalue()
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
+            if not le.datetime:
+                continue
             assert(le.datetime >= random_start and le.datetime <= random_end)
 
+
+    def test_from_to_stdin(self):
+
+        year = datetime.now().year
+        start = datetime(year, 8, 5, 20, 45)
+        end = datetime(year, 8, 5, 21, 01)
+        self.tool.is_stdin = True
+        self.tool.run('%s --from %s --to %s'%(self.logfile_path, start.strftime("%b %d %H:%M:%S"), end.strftime("%b %d %H:%M:%S")))
+        self.tool.is_stdin = False
+
+        output = sys.stdout.getvalue()
+        for line in output.splitlines():
+            le = LogEvent(line)
+            assert(le.datetime >= start and le.datetime <= end)
+
+
     def test_json(self):
         """ output with --json is in JSON format. """
         self.tool.run('%s --json'%self.logfile_path)
@@ -76,12 +98,18 @@ class TestMLogFilter(object):
             assert(line_dict)
             assert(type(line_dict) == dict)
 
-    def test_shorten(self):
+    def test_shorten_50(self):
         self.tool.run('%s --shorten 50'%self.logfile_path)
         output = sys.stdout.getvalue()
         for line in output.splitlines():
             assert(len(line) <= 50)
 
+    def test_shorten_default(self):
+        self.tool.run('%s --shorten'%self.logfile_path)
+        output = sys.stdout.getvalue()
+        for line in output.splitlines():
+            assert(len(line) <= 200)
+
     def test_merge_same(self):
         file_length = len(self.logfile)
         self.tool.run('%s %s'%(self.logfile_path, self.logfile_path))
@@ -89,7 +117,43 @@ class TestMLogFilter(object):
         lines = output.splitlines()
         assert len(lines) == 2*file_length
         for prev, next in zip(lines[:-1], lines[1:]):
-            assert LogEvent(prev).datetime <= LogEvent(next).datetime
+            prev_le = LogEvent(prev)
+            next_le = LogEvent(next)
+            if not prev_le.datetime or not next_le.datetime:
+                continue
+            assert prev_le.datetime <= next_le.datetime
+
+    def test_merge_markers(self):
+        file_length = len(self.logfile)
+        self.tool.run('%s %s --markers foo bar'%(self.logfile_path, self.logfile_path))
+        output = sys.stdout.getvalue()
+        lines = output.splitlines()
+        assert len([l for l in lines if l.startswith('foo')]) == file_length
+        assert len([l for l in lines if l.startswith('bar')]) == file_length
+
+    def test_merge_invalid_markers(self):
+        try:
+            self.tool.run('%s %s --markers foo bar baz'%(self.logfile_path, self.logfile_path))
+        except SystemExit as e:
+            assert 'Number of markers not the same' in e.message
+
+    def test_exclude(self):
+        file_length = len(self.logfile)
+        tool = MLogFilterTool()
+        tool.run('%s --slow 300' % self.logfile_path)
+
+        tool = MLogFilterTool()
+        tool.run('%s --slow 300 --exclude' % self.logfile_path)
+        output = sys.stdout.getvalue()
+        lines_total = len(output.splitlines())
+
+        assert lines_total == file_length
+
+
+    def test_end_reached(self):
+        self.tool.run('%s --from Jan 3015 --to +10min'%self.logfile_path)
+        output = sys.stdout.getvalue()
+        assert output.strip() == ''
 
 
     def test_human(self):
@@ -107,30 +171,77 @@ class TestMLogFilter(object):
         output = sys.stdout.getvalue()
         assert(len(output.splitlines()) > 0)
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
             assert(le.duration >= 145 and le.duration <= 500)
 
+    def test_scan(self):
+        # load tablescan logfile
+        scn_logfile_path = os.path.join(os.path.dirname(mtools.__file__), 'test/logfiles/', 'collscans.log')
+
+        self.tool.run('%s --scan' % scn_logfile_path)
+        output = sys.stdout.getvalue()
+        lines = output.splitlines()
+        assert len(lines) == 3
+
+    def test_accept_nodate(self):
+        self.tool.is_stdin = True
+        self.tool.run('%s --from Aug 5 2014 20:53:50 --to +5min'%self.logfile_path)
+        self.tool.is_stdin = False
+
+        output = sys.stdout.getvalue()
+        lines = output.splitlines()
+        assert any('=== a line without a datetime ===' in l for l in lines)
+
     def test_thread(self):
         self.tool.run('%s --thread initandlisten'%self.logfile_path)
         output = sys.stdout.getvalue()
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
             assert(le.thread == 'initandlisten')
 
+    def test_no_timestamp_format(self):
+        self.tool.run('%s --timestamp-format none --timezone 5'%self.logfile_path)
+        output = sys.stdout.getvalue()
+        for line in output.splitlines():
+            le = LogEvent(line)
+            if le.datetime:
+                assert le.datetime_format == 'ctime-pre2.4'
+
     def test_operation(self):
         self.tool.run('%s --operation insert'%self.logfile_path)
         output = sys.stdout.getvalue()
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
             assert(le.operation == 'insert')
 
+    def test_invalid_timezone_args(self):
+        try:
+            self.tool.run('%s --timezone 1 2 3'%self.logfile_path)
+        except SystemExit as e:
+            assert "Invalid number of timezone parameters" in e.message
+
+    def test_verbose(self):
+        self.tool.run('%s --slow --verbose'%self.logfile_path)
+        output = sys.stdout.getvalue()
+        lines = output.splitlines()
+        assert lines[0].startswith('command line arguments')
+        assert any( line.startswith('active filters: SlowFilter') for line in lines )
+
     def test_namespace(self):
         self.tool.run('%s --namespace local.oplog.rs'%self.logfile_path)
         output = sys.stdout.getvalue()
         for line in output.splitlines():
-            le =  LogEvent(line)
+            le = LogEvent(line)
             assert(le.namespace == 'local.oplog.rs')
 
+    def test_pattern(self):
+        # test that pattern is correctly parsed, reordered and compared to logevent pattern
+        self.tool.run('%s --pattern {ns:1,_id:1,host:1}'%self.logfile_path)
+        output = sys.stdout.getvalue()
+        for line in output.splitlines():
+            le = LogEvent(line)
+            assert(le.pattern == '{"_id": 1, "host": 1, "ns": 1}')
+
     def test_word(self):
         self.tool.run('%s --word lock'%self.logfile_path)
         output = sys.stdout.getvalue()
@@ -150,7 +261,7 @@ class TestMLogFilter(object):
         output = sys.stdout.getvalue()
         for line in output.splitlines():
             le =  LogEvent(line)
-            assert( 
+            assert(
                     (le.datetime >= event1 - padding and le.datetime <= event1 + padding) or
                     (le.datetime >= event2 - padding and le.datetime <= event2 + padding)
                   )
@@ -169,7 +280,7 @@ class TestMLogFilter(object):
         output = sys.stdout.getvalue()
         for line in output.splitlines():
             le =  LogEvent(line)
-            assert( 
+            assert(
                     (le.datetime >= event1 - duration1 - padding and le.datetime <= event1 - duration1 + padding) or
                     (le.datetime >= event2 - padding and le.datetime <= event2 + padding)
                   )
@@ -188,7 +299,7 @@ class TestMLogFilter(object):
         output = sys.stdout.getvalue()
         for line in output.splitlines():
             le =  LogEvent(line)
-            assert( 
+            assert(
                     (le.datetime >= event1 - duration1 - padding and le.datetime <= event1 + padding) or
                     (le.datetime >= event2 - padding and le.datetime <= event2 + padding)
                   )
@@ -225,4 +336,4 @@ class TestMLogFilter(object):
             assert line.startswith("2013-")
 
 
-# output = sys.stdout.getvalue().strip()
\ No newline at end of file
+# output = sys.stdout.getvalue().strip()
