@@ -4,6 +4,9 @@ import dateutil.parser
 import re
 import json
 
+from mtools.util.pattern import json2pattern
+
+
 class DateTimeEncoder(json.JSONEncoder):
     """ custom datetime encoder for json output. """
     def default(self, obj):
@@ -13,21 +16,21 @@ class DateTimeEncoder(json.JSONEncoder):
 
 
 class LogEvent(object):
-    """ LogEvent extracts information from a mongod/mongos log file line and 
+    """ LogEvent extracts information from a mongod/mongos log file line and
         stores the following properties/variables:
 
         line_str: the original line string
-        split_tokens: a list of string tokens after splitting line_str using 
+        split_tokens: a list of string tokens after splitting line_str using
                       whitespace as split points
-        datetime: a datetime object for the logevent. For logfiles created with 
+        datetime: a datetime object for the logevent. For logfiles created with
                   version 2.4+, it also contains micro-seconds
         duration: the duration of a timed operation in ms
         thread: the thread name (e.g. "conn1234") as string
         operation: insert, update, remove, query, command, getmore, None
         namespace: the namespace of the operation, or None
-        
+
         Certain operations also add the number of affected/scanned documents.
-        If applicable, the following variables are also set, otherwise the 
+        If applicable, the following variables are also set, otherwise the
         default is None: nscanned, ntoreturn, nreturned, ninserted, nupdated
 
         For performance reason, all fields are evaluated lazily upon first
@@ -44,7 +47,7 @@ class LogEvent(object):
 
 
     def __init__(self, doc_or_str):
-        self.year_rollover = False
+        self._year_rollover = False
 
         if isinstance(doc_or_str, str):
             # create from string, remove line breaks at end of _line_str
@@ -75,13 +78,13 @@ class LogEvent(object):
 
         self._thread_calculated = False
         self._thread = None
-        self._thread_offset = None
 
         self._operation_calculated = False
         self._operation = None
         self._namespace = None
 
         self._pattern = None
+        self._sort_pattern = None
 
         self._counters_calculated = False
         self._nscanned = None
@@ -133,15 +136,20 @@ class LogEvent(object):
     @property
     def duration(self):
         """ calculate duration if available (lazy) """
-        
+
         if not self._duration_calculated:
             self._duration_calculated = True
 
-            split_tokens = self.split_tokens
+            # split_tokens = self.split_tokens
+            line_str = self.line_str
 
-            if len(split_tokens) > 0 and split_tokens[-1].endswith('ms'):
+            if line_str and line_str.endswith('ms'):
                 try:
-                    self._duration = int((split_tokens[-1][:-2]).replace(',',''))
+                    # find duration from end
+                    space_pos = line_str.rfind(" ")
+                    if space_pos == -1:
+                        return
+                    self._duration = int(line_str[line_str.rfind(" ")+1:-2].replace(',',''))
                 except ValueError:
                     self._duration = None
             elif "flushing" in self.line_str:
@@ -180,8 +188,8 @@ class LogEvent(object):
 
         return self._datetime
 
-    
-    @property 
+
+    @property
     def datetime_format(self):
         if not self._datetime_calculated:
             _ = self.datetime
@@ -189,9 +197,34 @@ class LogEvent(object):
         return self._datetime_format
 
 
+    @property
+    def datetime_nextpos(self):
+        if self._datetime_nextpos == None and not self._datetime_calculated:
+            _ = self.datetime
+        return self._datetime_nextpos
+
+
+    def set_datetime_hint(self, format, nextpos, rollover):
+        self._datetime_format = format
+        self._datetime_nextpos = nextpos
+        self._year_rollover = rollover
+
+        # fast check if timezone changed. if it has, trigger datetime evaluation
+        if format.startswith('ctime'):
+            if len(self.split_tokens) < 4 or self.split_tokens[self._datetime_nextpos-4] not in self.weekdays:
+                _ = self.datetime
+                return False
+            return True
+        else:
+            if not self.split_tokens[self._datetime_nextpos-1][0].isdigit():
+                _ = self.datetime
+                return False
+            return True
+
+
     def _match_datetime_pattern(self, tokens):
-        """ Helper method that takes a list of tokens and tries to match 
-            the datetime pattern at the beginning of the token list. 
+        """ Helper method that takes a list of tokens and tries to match
+            the datetime pattern at the beginning of the token list.
 
             There are several formats that this method needs to understand
             and distinguish between (see MongoDB's SERVER-7965):
@@ -212,13 +245,13 @@ class LogEvent(object):
                 assume_iso8601_format = True
 
         if assume_iso8601_format:
-            # sanity check, because the dateutil parser could interpret 
+            # sanity check, because the dateutil parser could interpret
             # any numbers as a valid date
             if not re.match(r'\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.\d{3}', \
                             tokens[0]):
                 return None
 
-            # convinced that this is a ISO-8601 format, the dateutil parser 
+            # convinced that this is a ISO-8601 format, the dateutil parser
             # will do the rest
             dt = dateutil.parser.parse(tokens[0])
             self._datetime_format = "iso8601-utc" \
@@ -227,12 +260,14 @@ class LogEvent(object):
         else:
             # assume current year unless self.year_rollover is set (from LogFile)
             year = datetime.now().year
-            if self.year_rollover:
-                year -= 1
             dt = dateutil.parser.parse(' '.join(tokens[:4]), default=datetime(year, 1, 1))
+
             if dt.tzinfo == None:
                 dt = dt.replace(tzinfo=tzutc())
-                
+
+            if self._year_rollover and dt > self._year_rollover:
+                dt = dt.replace(year=year-1)
+
             self._datetime_format = "ctime" \
                 if '.' in tokens[3] else "ctime-pre2.4"
 
@@ -248,23 +283,20 @@ class LogEvent(object):
 
             split_tokens = self.split_tokens
 
-            # force evaluation of datetime to get access to datetime_offset
-            if self.datetime:
-                if len(split_tokens) <= self._datetime_nextpos:
-                    return None
+            if not self.datetime_nextpos or len(split_tokens) <= self.datetime_nextpos:
+                return None
+
+            connection_token = split_tokens[self.datetime_nextpos]
+            match = re.match(r'^\[([^\]]*)\]$', connection_token)
+            if match:
+                self._thread = match.group(1)
 
-                connection_token = split_tokens[self._datetime_nextpos]
-                match = re.match(r'^\[([^\]]*)\]$', connection_token)
-                if match:
-                    self._thread = match.group(1)
-                    self._thread_offset = self._datetime_nextpos
-  
         return self._thread
 
 
     @property
     def operation(self):
-        """ extract operation (query, insert, update, remove, getmore, command) 
+        """ extract operation (query, insert, update, remove, getmore, command)
             if available (lazy) """
 
         if not self._operation_calculated:
@@ -286,22 +318,25 @@ class LogEvent(object):
 
 
     def _extract_operation_and_namespace(self):
-        """ Helper method to extract both operation and namespace from a 
+        """ Helper method to extract both operation and namespace from a
             logevent. It doesn't make sense to only extract one as they
             appear back to back in the token list.
         """
 
         split_tokens = self.split_tokens
 
-        # trigger datetime evaluation to get access to offset
-        if self.datetime:
-            if len(split_tokens) <= self._datetime_nextpos + 2:
-                return
-            op = split_tokens[self._datetime_nextpos + 1]
+        if not self._datetime_nextpos:
+            # force evaluation of datetime to get access to datetime_offset
+            _ = self.datetime
 
-            if op in ['query', 'insert', 'update', 'remove', 'getmore', 'command']:
-                self._operation = op
-                self._namespace = split_tokens[self._datetime_nextpos + 2]
+        if not self._datetime_nextpos or len(split_tokens) <= self._datetime_nextpos + 2:
+            return
+
+        op = split_tokens[self._datetime_nextpos + 1]
+
+        if op in ['query', 'insert', 'update', 'remove', 'getmore', 'command']:
+            self._operation = op
+            self._namespace = split_tokens[self._datetime_nextpos + 2]
 
 
 
@@ -312,30 +347,24 @@ class LogEvent(object):
         if not self._pattern:
 
             # trigger evaluation of operation
-            if self.operation:
-                # get start of json query pattern
-                start_idx = self.line_str.rfind(' query: ')
-                if start_idx == -1:
-                    # no query pattern found
-                    return
-
-                stop_idx = 0
-                brace_counter = 0
-                search_str = self.line_str[start_idx+8:]
-
-                for match in re.finditer(r'{|}', search_str):
-                    stop_idx = match.start()
-                    if search_str[stop_idx] == '{':
-                        brace_counter += 1
-                    else:
-                        brace_counter -= 1
-                    if brace_counter == 0:
-                        break
-                search_str = search_str[:stop_idx+1].strip()
-                self._pattern = '{' + ': 1, '.join(sorted(re.findall(r' (\w+): ', search_str))) + ': 1}'
+            if self.operation in ['query', 'getmore', 'update', 'remove']:
+                self._pattern = self._find_pattern('query: ')
 
         return self._pattern
 
+    
+    @property
+    def sort_pattern(self):
+        """ extract query pattern from operations """
+
+        if not self._sort_pattern:
+
+            # trigger evaluation of operation
+            if self.operation in ['query', 'getmore']:
+                self._sort_pattern = self._find_pattern('orderby: ')
+
+        return self._sort_pattern
+
 
     @property
     def nscanned(self):
@@ -432,7 +461,7 @@ class LogEvent(object):
 
     def _extract_counters(self):
         """ Helper method to extract counters like nscanned, nreturned, etc.
-            from the logevent. 
+            from the logevent.
         """
 
         # extract counters (if present)
@@ -443,7 +472,7 @@ class LogEvent(object):
 
         # trigger thread evaluation to get access to offset
         if self.thread:
-            for t, token in enumerate(split_tokens[self._thread_offset+2:]):
+            for t, token in enumerate(split_tokens[self.datetime_nextpos+2:]):
                 for counter in counters:
                     if token.startswith('%s:'%counter):
                         try:
@@ -453,17 +482,17 @@ class LogEvent(object):
                             # https://jira.mongodb.org/browse/SERVER-10101
                             if counter == 'numYields' and token.startswith('numYields'):
                                 try:
-                                    self._numYields = int((split_tokens[t+1+self._thread_offset+2]).replace(',', ''))
+                                    self._numYields = int((split_tokens[t+1+self.datetime_nextpos+2]).replace(',', ''))
                                 except ValueError:
                                     pass
                         # token not parsable, skip
                         break
-                    
+
 
 
     def parse_all(self):
         """ triggers the extraction of all information, which would usually
-            just be evaluated lazily. 
+            just be evaluated lazily.
         """
         tokens = self.split_tokens
         duration = self.duration
@@ -483,12 +512,42 @@ class LogEvent(object):
         r = self.r
 
 
+    def _find_pattern(self, trigger):
+
+        # get start of json query pattern
+        start_idx = self.line_str.rfind(trigger)
+        if start_idx == -1:
+            # no query pattern found
+            return None
+
+        stop_idx = 0
+        brace_counter = 0
+        search_str = self.line_str[start_idx+len(trigger):]
+
+        for match in re.finditer(r'{|}', search_str):
+            stop_idx = match.start()
+            if search_str[stop_idx] == '{':
+                brace_counter += 1
+            else:
+                brace_counter -= 1
+            if brace_counter == 0:
+                break
+        search_str = search_str[:stop_idx+1].strip()
+        if search_str:
+            return json2pattern(search_str)
+        else:
+            return None
+
+
+
     def _reformat_timestamp(self, format, force=False):
         if format not in ['ctime', 'ctime-pre2.4', 'iso8601-utc', 'iso8601-local']:
-            raise ValueError('invalid datetime format %s, choose from ctime, ctime-pre2.4, iso8601-utc, iso8601-local.') 
+            raise ValueError('invalid datetime format %s, choose from ctime, ctime-pre2.4, iso8601-utc, iso8601-local.')
 
         if (self.datetime_format == None or (self.datetime_format == format and self._datetime_str != '')) and not force:
             return
+        elif self.datetime == None:
+            return
         elif format.startswith('ctime'):
             dt_string = self.weekdays[self.datetime.weekday()] + ' ' + self.datetime.strftime("%b %d %H:%M:%S")
             # remove zero-padding from day number
@@ -508,7 +567,7 @@ class LogEvent(object):
         elif format == 'iso8601-utc':
             if self.datetime.utcoffset():
                 dt_string = self.datetime.astimezone(tzutc()).strftime("%Y-%m-%dT%H:%M:%S")
-            else: 
+            else:
                 dt_string = self.datetime.strftime("%Y-%m-%dT%H:%M:%S")
             dt_string += '.' + str(int(self.datetime.microsecond * 1000)).zfill(3)[:3] + 'Z'
 
@@ -535,9 +594,9 @@ class LogEvent(object):
             if value != None:
                 output[label] = value
 
-        return output     
+        return output
+
 
-    
     def to_json(self, labels=None):
         """ converts LogEvent object to valid JSON. """
         output = self.to_dict(labels)
@@ -563,14 +622,27 @@ class LogEvent(object):
 
         self._thread_calculated = True
         self._thread = doc['thread']
-        self._thread_offset = None
 
         self._operation_calculated = True
         self._operation = doc[u'op']
         self._namespace = doc[u'ns']
 
-        # TODO: pattern parser also for system.profile events
-        self._pattern = '{}'
+        # query pattern for system.profile events, all three cases (see SERVER-13245)
+        if 'query' in doc:
+            if 'query' in doc['query'] and isinstance(doc['query']['query'], dict):
+                self._pattern = str(doc['query']['query']).replace("'", '"')
+            elif '$query' in doc['query']:
+                self._pattern = str(doc['query']['$query']).replace("'", '"')
+            else:
+                self._pattern = str(doc['query']).replace("'", '"')
+
+            # sort pattern
+            if 'orderby' in doc['query'] and isinstance(doc['query']['orderby'], dict):
+                self._sort_pattern = str(doc['query']['orderby']).replace("'", '"')    
+            elif '$orderby' in doc['query']:
+                self._sort_pattern = str(doc['query']['$orderby']).replace("'", '"')
+            else: 
+                self._sort_pattern = None
 
         self._counters_calculated = True
         self._nscanned = doc[u'nscanned'] if 'nscanned' in doc else None
@@ -598,9 +670,7 @@ class LogEvent(object):
         scanned = 'nscanned:%i'%self._nscanned if 'nscanned' in doc else ''
         yields = 'numYields:%i'%self._numYields if 'numYield' in doc else ''
         locks = 'w:%i' % self.w if self.w != None else 'r:%i' % self.r
-        duration = '%ims' % self.duration if self.duration != None else ''    
+        duration = '%ims' % self.duration if self.duration != None else ''
 
         self._line_str = "[{thread}] {operation} {namespace} {payload} {scanned} {yields} locks(micros) {locks} {duration}".format(
             datetime=self.datetime, thread=self.thread, operation=self.operation, namespace=self.namespace, payload=payload, scanned=scanned, yields=yields, locks=locks, duration=duration)
-
-
