@@ -31,8 +31,6 @@ class LogFile(InputSource):
     @property
     def start(self):
         """ lazy evaluation of start and end of logfile. Returns None for stdin input currently. """
-        if self.from_stdin:
-            return None
         if not self._start:
             self._calculate_bounds()
         return self._start
@@ -40,8 +38,6 @@ class LogFile(InputSource):
     @property
     def end(self):
         """ lazy evaluation of start and end of logfile. Returns None for stdin input currently. """
-        if self.from_stdin:
-            return None
         if not self._end:
             self._calculate_bounds()
         return self._end
@@ -102,44 +98,64 @@ class LogFile(InputSource):
         return versions
 
     def next(self):
-        """ makes LogFiles iterators. """
-        line = self.filehandle.next()
+        """ get next line, adjust for year rollover and hint datetime format. """
+
+        # use readline here because next() iterator uses internal readahead buffer so seek position is wrong
+        line = self.filehandle.readline()
+        if line == '':
+            raise StopIteration
+        line = line.rstrip('\n')
+
         le = LogEvent(line)
-        # adjust for year rollover if necessary
-        if self._year_rollover:
-            self.adjust_year_rollover(le)
+        
+        # hint format and nextpos from previous line
+        if self._datetime_format and self._datetime_nextpos != None:
+            ret = le.set_datetime_hint(self._datetime_format, self._datetime_nextpos, self.year_rollover)
+            if not ret:
+                # logevent indicates timestamp format has changed, invalidate hint info
+                self._datetime_format = None
+                self._datetime_nextpos = None
+        elif le.datetime:
+            # print "not hinting"
+            # gather new hint info from another logevent
+            self._datetime_format = le.datetime_format
+            self._datetime_nextpos = le._datetime_nextpos  
+
         return le
 
     def __iter__(self):
         """ iteration over LogFile object will return a LogEvent object for each line (generator) """
+        le = None
+        
+        while True:
+            try:
+                le = self.next()
+            except StopIteration as e:
+                # end of log file, get end date
+                if not self.end and self.from_stdin:
+                    if le and le.datetime:
+                        self._end = le.datetime
+
+                # future iterations start from the beginning
+                if not self.from_stdin:
+                    self.filehandle.seek(0)
+                
+                # now raise StopIteration exception
+                raise e
+
+            # get start date for stdin input
+            if not self.start and self.from_stdin:
+                if le and le.datetime:
+                    self._start = le.datetime
 
-        for line in self.filehandle:
-            le = LogEvent(line)
-            # adjust for year rollover if necessary
-            if self._year_rollover:
-                self.adjust_year_rollover(le)
             yield le
 
-        # future iterations start from the beginning
-        if not self.from_stdin:
-            self.filehandle.seek(0)
-
 
     def __len__(self):
         """ return the number of lines in a log file. """
         return self.num_lines
 
 
-    def adjust_year_rollover(self, logevent):
-        """ checks if logevent needs adjustment due to year rollover in logfile. """
-        if self._year_rollover and logevent.datetime and logevent.datetime > self.end:
-            # roll back year now and set year_rollover flag for future conversions
-            logevent.year_rollover = True
-            logevent._datetime = logevent._datetime.replace(year=logevent._datetime.year - 1)
-            logevent._reformat_timestamp(self._datetime_format, force=True)
-        return logevent
-
-
     def _iterate_lines(self):
         """ count number of lines (can be expensive). """
         self._num_lines = 0
@@ -182,28 +198,27 @@ class LogFile(InputSource):
         # get start datetime 
         for line in self.filehandle:
             logevent = LogEvent(line)
-            date = logevent.datetime
-            if date:
-                self._start = date
+            if logevent.datetime:
+                self._start = logevent.datetime
                 self._datetime_format = logevent.datetime_format
+                self._datetime_nextpos = logevent._datetime_nextpos
                 break
 
-        # get end datetime (lines are at most 10k, go back 15k at most to make sure)
+        # get end datetime (lines are at most 10k, go back 30k at most to make sure we catch one)
         self.filehandle.seek(0, 2)
         self._filesize = self.filehandle.tell()
-        self.filehandle.seek(-min(self._filesize, 15000), 2)
+        self.filehandle.seek(-min(self._filesize, 30000), 2)
 
         for line in reversed(self.filehandle.readlines()):
             logevent = LogEvent(line)
-            date = logevent.datetime
-            if date:
-                self._end = date
+            if logevent.datetime:
+                self._end = logevent.datetime
                 break
 
         # if there was a roll-over, subtract 1 year from start time
         if self._end < self._start:
             self._start = self._start.replace(year=self._start.year-1)
-            self._year_rollover = True
+            self._year_rollover = self._end
         else:
             self._year_rollover = False
 
@@ -232,37 +247,31 @@ class LogFile(InputSource):
 
         # move back to last newline char
         if newline_pos == -1:
+            self.filehandle.seek(0)
+            return self.next()
+
+        self.filehandle.seek(newline_pos - jump_back + 1, 1)
+
+        # roll forward until we found a line with a datetime
+        try:
+            logevent = self.next()
+            while not logevent.datetime:
+                logevent = self.next()
+
+            return logevent
+        except StopIteration:
+            # reached end of file
             return None
 
-        self.filehandle.seek(newline_pos - jump_back, 1)
-
-        while line != '':
-            line = self.filehandle.readline()
-            logevent = LogEvent(line)
-            if logevent.datetime:
-                # check for year rollover
-                self.adjust_year_rollover(logevent)
-                return logevent
-
-            # to avoid infinite loops, quit here if previous line not found
-            if prev:
-                return None
-
 
     def fast_forward(self, start_dt):
         """ Fast-forward a log file to the given start_dt datetime object using binary search.
             Only fast for files. Streams need to be forwarded manually, and it will miss the 
             first line that would otherwise match (as it consumes the log line). 
         """
-
         if self.from_stdin:
             # skip lines until start_dt is reached
-            le = None
-            while not (le and le.datetime and le.datetime >= start_dt):
-                line = self.filehandle.next()
-                # can't check for year rollover in a stream
-                le = LogEvent(line)
-
+            return
 
         else:
             # fast bisection path
@@ -270,7 +279,15 @@ class LogFile(InputSource):
             max_mark = self.filesize
             step_size = max_mark
 
-            le =  None
+            # check if start_dt is already smaller than first datetime
+            self.filehandle.seek(0)
+            le = self.next()
+            if le.datetime and le.datetime >= start_dt:
+                self.filehandle.seek(0)
+                return
+
+            le = None
+            self.filehandle.seek(0)
 
             # search for lower bound
             while abs(step_size) > 100:
