@@ -4,16 +4,17 @@ import sys
 import commands
 import subprocess
 import cPickle
-from mtools.util import OrderedDict
 from collections import defaultdict
 
 from mtools.util.logcodeline import LogCodeLine
 
 from pymongo import MongoClient
+from pymongo.errors import ConnectionFailure
 
 
 mongodb_path = "/Users/tr/Documents/code/mongo/"
 git_path = "/usr/bin/git"
+pattern_id = 0
 
 def source_files(mongodb_path):
     for root, dirs, files in os.walk(mongodb_path):
@@ -50,6 +51,9 @@ def get_all_versions():
     # remove developer versions
     versions = [v for v in versions if re.search(r'\.[02468]\.', v)]
 
+    # add master logs
+    versions.append('master')
+
     return versions
 
 
@@ -76,6 +80,7 @@ def output_verbose(version, filename, lineno, line, statement, matches, accepted
 
 
 def extract_logs(log_code_lines, current_version):
+    global pattern_id
     log_templates = set()
     log_triggers = [" log(", " LOG(", " LOGSOME", " warning()", " error()", " out()", " problem()"]
 
@@ -175,18 +180,26 @@ def extract_logs(log_code_lines, current_version):
                     # output_verbose(current_version, filename, lineno, line, statement, matches, False, "contains 'query:'")
                     continue
 
-                loglevel = re.match(r'LOG\(([0-9])\)', statement)
+                loglevel = re.search(r'LOG\(\s*([0-9])\s*\)', line)
                 if loglevel:
                     loglevel = int(loglevel.group(1))
 
-                matches = tuple(matches)
+                if trigger == ' log(':
+                    loglevel = 0
+
+                pattern = tuple(matches)
 
                 # add to log_code_lines dict
-                if not matches in log_code_lines:
-                    log_code_lines[matches] = LogCodeLine(matches)
+                if not pattern in log_code_lines:
+                    log_code_lines[pattern] = LogCodeLine(pattern, pattern_id)
+                    pattern_id += 1
 
-                log_code_lines[matches].addMatch(current_version, filename, lineno, loglevel, trigger)
-                log_templates.add(matches)
+                # clean up values
+                lineno = lineno+1
+                trigger = trigger.strip()
+
+                log_code_lines[pattern].addMatch(current_version, filename, lineno, loglevel, trigger)
+                log_templates.add(pattern)
 
                 # output_verbose(current_version, filename, lineno, line, statement, matches, True, "OK")
 
@@ -233,15 +246,21 @@ if __name__ == '__main__':
     #     print " <var> ".join(l), "found in:", ", ".join(logs_versions[l])
 
     # write out to mongodb
-    # mc = MongoClient()
-    for pattern_id, pattern in enumerate(log_code_lines):
+    write_to_db = True
+    try:
+        mc = MongoClient()
+        mc['log2code']['instances'].drop()
+    except ConnectionFailure:
+        write_to_db = False
+
+    for pattern in log_code_lines:
         lcl = log_code_lines[pattern]
 
         first_token = pattern[0]
         split_words = first_token.split()
 
         instance = {
-            '_id': pattern_id,
+            '_id': lcl.pattern_id,
             'pattern': list(lcl.pattern),
             'first_word': split_words[0],
             'matches': []
@@ -258,7 +277,8 @@ if __name__ == '__main__':
                     'trigger': occ[3]
                 })
         
-        # mc['log2code']['instances'].update({'pattern': instance['pattern']}, instance, upsert=True)
+        if write_to_db:
+            mc['log2code']['instances'].update({'pattern': instance['pattern']}, instance, upsert=True)
 
     cPickle.dump((versions, logs_versions, logs_by_word, log_code_lines), open('log2code.pickle', 'wb'), -1)
 
